/**
 * Copyright 2005 Jason LaDere
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Originally based on Project64 code.
 *
 */

/*
 * Video.java
 *
 * Created on September 3, 2006, 3:14 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package jario64;

import java.nio.ByteBuffer;
import java.util.Properties;
import javax.swing.JFrame;
import plugin.GfxPlugin;

/**
 *
 * @author Jason
 */
public class Video {
    
    public static final String GFX_PLUGIN = "GFX_PLUGIN";
    
//    // regVI Video Interface (VI) Registers
//    public static final int VI_STATUS_REG=			0;
//    public static final int VI_ORIGIN_REG= 			1;
//    public static final int VI_WIDTH_REG= 			2;
//    public static final int VI_INTR_REG=  			3;
//    public static final int VI_CURRENT_REG=                     4;
//    public static final int VI_BURST_REG=  			5;
//    public static final int VI_V_SYNC_REG= 			6;
//    public static final int VI_H_SYNC_REG= 			7;
//    public static final int VI_LEAP_REG=  			8;
//    public static final int VI_H_START_REG=                     9;
//    public static final int VI_V_START_REG=                     10;
//    public static final int VI_V_BURST_REG=			11;
//    public static final int VI_X_SCALE_REG=			12;
//    public static final int VI_Y_SCALE_REG=			13;
    
    private static final int NUM_FRAMES = 7;
    
    // used by Main
    public long millisInGfx;
    
    // Video Interface (VI) Registers
    // used by Memory
    public int[] regVI = new int[14];
    
    // used by Main, Memory
    public GfxPlugin gfxPlugin;
    
    private Runnable checkInterrupts;
    private ByteBuffer rDram;
    private ByteBuffer dMem;
    private ByteBuffer iMem;
    private int[] regMI;
    private int[] regDPC;
    private JFrame hWnd;
    private long lastFrame;
    private long lastTime;
    private long[] frames = new long[NUM_FRAMES];
    private int currentFrame;
    private long frequency;
    private int oldViVsyncReg = 0;
    private int viIntrTime = 500000;
    private int viFieldNumber;
    private int halfLine;
    private Cop0 cop0; // for timer
    private boolean frameLimit;
    
    public Runnable timerInterrupt = new Runnable() {
        public void run() {
            regMI[Registers.MI_INTR_REG] |= Registers.MI_INTR_VI;
            checkInterrupts.run();
            if (gfxPlugin != null) {
                cop0.update();
                long time = System.currentTimeMillis();
//                cop0.changeTimer(Cop0.VI_TIMER, cop0.timer + cop0.nextTimer[Cop0.VI_TIMER] + gfxPlugin.updateScreen());
                cop0.changeTimer(Cop0.VI_TIMER, cop0.timer + cop0.nextTimer[Cop0.VI_TIMER] + refreshScreen());
                millisInGfx += (System.currentTimeMillis() - time);
            }
        }
    };
    
    /** Creates a new instance of Video */
    public Video(Runnable checkInterrupts, Registers regs, ByteBuffer rDram, ByteBuffer dMem, ByteBuffer iMem) {
        this.checkInterrupts = checkInterrupts;
        this.rDram = rDram;
        this.dMem = dMem;
        this.iMem = iMem;
        this.regMI = regs.regMI;
        this.regDPC = regs.regDPC;
        frequency = 1000;
        currentFrame = 0;
        viFieldNumber = 0;
    }
    
    // called by Main
    public void setTimer(Cop0 cop0) {
        this.cop0 = cop0;
    }

    // called by Main
    public void setFrameLimit(boolean frameLimit) {
        this.frameLimit = frameLimit;
    }
    
    // called by Main
    public boolean setupPlugin(JFrame hWnd, Properties cfg) {
        //shutdownPlugins();
        this.hWnd = hWnd;
        String gfx_plugin = cfg.getProperty(GFX_PLUGIN, "DEFAULT_GFX_PLUGIN");
        try {
            Class c = Class.forName(gfx_plugin);
            gfxPlugin = (GfxPlugin)c.newInstance();
        } catch (Exception ex) {
            System.err.println("No gfx plugin loaded.");
            ex.printStackTrace();
            return false;
        }
        
        GfxPlugin.GfxInfo gfxInfo = new GfxPlugin.GfxInfo();
        
        gfxInfo.memoryBswaped = true;
        gfxInfo.checkInterrupts = checkInterrupts;
        gfxInfo.hWnd = hWnd;
        gfxInfo.rdram = rDram;
        gfxInfo.dmem = dMem;
        gfxInfo.imem = iMem;
        
        gfxInfo.MI_INTR_REG = Registers.MI_INTR_REG;
        gfxInfo.miRegisters = regMI;
        gfxInfo.DPC_START_REG = Registers.DPC_START_REG;
        gfxInfo.DPC_END_REG = Registers.DPC_END_REG;
        gfxInfo.DPC_CURRENT_REG = Registers.DPC_CURRENT_REG;
        gfxInfo.DPC_STATUS_REG = Registers.DPC_STATUS_REG;
        gfxInfo.DPC_CLOCK_REG = Registers.DPC_CLOCK_REG;
        gfxInfo.DPC_BUFBUSY_REG = Registers.DPC_BUFBUSY_REG;
        gfxInfo.DPC_PIPEBUSY_REG = Registers.DPC_PIPEBUSY_REG;
        gfxInfo.DPC_TMEM_REG = Registers.DPC_TMEM_REG;
        gfxInfo.dpcRegisters = regDPC;
//        gfxInfo.VI_STATUS_REG = VI_STATUS_REG;
//        gfxInfo.VI_ORIGIN_REG = VI_ORIGIN_REG;
//        gfxInfo.VI_WIDTH_REG = VI_WIDTH_REG;
//        gfxInfo.VI_INTR_REG = VI_INTR_REG;
//        gfxInfo.VI_V_CURRENT_LINE_REG = VI_CURRENT_REG;
////        gfxInfo.VI_TIMING_REG = VI_TIMING_REG;
//        gfxInfo.VI_TIMING_REG = VI_BURST_REG;
//        gfxInfo.VI_V_SYNC_REG = VI_V_SYNC_REG;
//        gfxInfo.VI_H_SYNC_REG = VI_H_SYNC_REG;
//        gfxInfo.VI_LEAP_REG = VI_LEAP_REG;
//        gfxInfo.VI_H_START_REG = VI_H_START_REG;
//        gfxInfo.VI_V_START_REG = VI_V_START_REG;
//        gfxInfo.VI_V_BURST_REG = VI_V_BURST_REG;
//        gfxInfo.VI_X_SCALE_REG = VI_X_SCALE_REG;
//        gfxInfo.VI_Y_SCALE_REG = VI_Y_SCALE_REG;
        gfxInfo.viRegisters = regVI;
        
        if (!gfxPlugin.initiateGfx(gfxInfo) ) {
            System.err.println("Failed to Initilize Graphics!");
            return false;
        }
        return true;
    }
    
    private int refreshScreen(){
        //long time = System.currentTimeMillis();
        
        if (oldViVsyncReg != regVI[gfxPlugin.VI_V_SYNC_REG]) {
            if (regVI[gfxPlugin.VI_V_SYNC_REG] == 0) {
                viIntrTime = 500000;
            } else {
                viIntrTime = (regVI[gfxPlugin.VI_V_SYNC_REG] + 1) * 1500;
                if ((regVI[gfxPlugin.VI_V_SYNC_REG] % 1) != 0) {
                    viIntrTime -= 38;
                }
            }
        }
        if ((regVI[gfxPlugin.VI_STATUS_REG] & 0x10) != 0) {
            if (viFieldNumber == 0) {
                viFieldNumber = 1;
            } else {
                viFieldNumber = 0;
            }
        } else {
            viFieldNumber = 0;
        }

        if ((currentFrame & 7) == 0) {
            long time = System.currentTimeMillis();
            frames[(currentFrame >> 3) % NUM_FRAMES] = time - lastFrame;
            lastFrame = time;
            if (currentFrame > (NUM_FRAMES << 3)) {
                long total = 0;
                for (int count=0; count<NUM_FRAMES; count++)
                    total += frames[count];
                hWnd.setTitle(String.format("FPS: %.2f", frequency / ((double)total / (NUM_FRAMES << 3))));
            } else {
                hWnd.setTitle("FPS: -.--");
            }
        }
        currentFrame += 1;
        
        if (gfxPlugin != null)
            gfxPlugin.updateScreen();
        
//        millisInGfx += (System.currentTimeMillis() - time);

        // new frame limiter
        if (frameLimit) {
            while (lastTime + 15L > System.currentTimeMillis()) { }
            lastTime = System.currentTimeMillis();
        }
        
        return viIntrTime;
    }
    
//    // called by Memory
//    public int updateCurrentHalfLine(int timer) {
//        //cop0.update();
//        if (timer < 0)
//            return 0;
//        int halfLine = (timer / 1500);
//        halfLine &= ~1;
//        halfLine += viFieldNumber;
//        //System.out.println("HALFLINE: " + halfLine);
//        return halfLine;
//    }
    
    public int updateCurrentHalfLine(int timer) {
        if (timer < 0)
            return 0;
        int halfLine = (timer / 1500);
        halfLine &= ~1;
        halfLine += viFieldNumber;
        return halfLine;
    }
    
//    private void shutdownPlugins() {
//        if (gfxPlugin != null) { gfxPlugin.CloseDLL(); }
//    }
    
}
