/**
 * Copyright 2005 Jason LaDere
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Originally based on Project64 code.
 *
 */

/*
 * DebugCpu.java
 *
 * Created on November 15, 2005, 10:00 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package jario64.debug;

import jario64.Cpu;
import jario64.Main;
import jario64.Memory;
import jario64.Registers;
import static jario64.Cop1.FSTATUS_REGISTER;

/**
 *
 * @author Jason
 */
public class DebugCpu extends Cpu {
    
    public String getITypeDebug(String name, long rs, long rt, int immediate) {
        return String.format("%X:%s:%d:%d:%X %X:%X:%X ", pc, name, rs, rt, immediate, rs, rt, immediate);
    }
    
    public String getRTypeDebug(String name, long rs, long rt, long rd) {
        return String.format("%X:%s:%d:%d:%d:%d %X:%X:%X ", pc, name, rs, rt, rd, sa, rs, rt, rd);
    }
    
    public String getJTypeDebug(String name, int target) {
        return String.format("%X:%s:%X %X ", pc, name, target, target);
    }
    
    /** Creates a new instance of InterpreterOps */
    public DebugCpu(Registers regs, Memory mem) {
        super(regs, mem);
    }
    
    public void printCpuState() {
        System.out.printf("CPU State:\n");
        System.out.printf("PC=%x\n", pc);
        for (int j=0; j<16; j++)
            System.out.printf("reg[%2d]:%8x%8x        reg[%d]:%8x%8x\n",
                    j,
                    (int)(GPR[j] >> 32),
                    (int)(GPR[j]),
                    j+16,
                    (int)(GPR[j+16] >> 32),
                    (int)(GPR[j+16]));
        System.out.printf("hi:%8x%8x        lo:%8x%8x\n",
                (int)(HI >> 32),
                (int)(HI),
                (int)(LO >> 32),
                (int)(LO));
    }
    
    /************************* OpCode functions *************************/
    public Runnable r4300i_J_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_J_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getJTypeDebug("J", target << 2));
            
            r4300i_J.run();
            
            if (Debug.DEBUG_J_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation, GPR[31]);
        }
    };
    
    public Runnable r4300i_JAL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_J_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getJTypeDebug("JAL", target << 2));
            
            r4300i_JAL.run();
            
            if (Debug.DEBUG_J_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X 31=%X\n", jumpToLocation, GPR[31]);
        }
    };
    
    public Runnable r4300i_BEQ_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BEQ", GPR[rs], GPR[rt], (int)((offset << 2) + 4)));
            
            r4300i_BEQ.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BNE_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BNE", GPR[rs], GPR[rt], (int)((offset << 2) + 4)));
            
            r4300i_BNE.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BLEZ_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BLEZ", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_BLEZ.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BGTZ_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BGTZ", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_BGTZ.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_ADDI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("ADDI", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_ADDI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_ADDIU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("ADDIU", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_ADDIU.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_SLTI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("SLTI", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_SLTI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_SLTIU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("SLTIU", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_SLTIU.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_ANDI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("ANDI", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_ANDI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_ORI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("ORI", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_ORI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_XORI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("XORI", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_XORI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_LUI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("LUI", 0, GPR[rt], (int)(offset << 16)));
            
            r4300i_LUI.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_BEQL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BEQL", GPR[rs], GPR[rt], (int)((offset << 2) + 4)));
            
            r4300i_BEQL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BNEL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BNEL", GPR[rs], GPR[rt], (int)((offset << 2) + 4)));
            
            r4300i_BNEL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BLEZL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BLEZL", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_BLEZL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_BGTZL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BGTZL", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_BGTZL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_DADDIU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getITypeDebug("DADDIU", GPR[rs], GPR[rt], (int)offset));
            
            r4300i_DADDIU.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_LDL_Debug = new Runnable() {
        public void run() {
            r4300i_LDL.run();
        }
    };
    
    public Runnable r4300i_LDR_Debug = new Runnable() {
        public void run() {
            r4300i_LDR.run();
        }
    };
    
    public Runnable r4300i_LB_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LB", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LB.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LH_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LH", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LH.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LWL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LWL", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LWL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_LW_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LW", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LW.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LBU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LBU", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LBU.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LHU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LHU", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LHU.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LWR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LWR", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LWR.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_LWU_Debug = new Runnable() {
        public void run() {
            r4300i_LWU.run();
        }
    };
    
    public Runnable r4300i_SB_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SB", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SB.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), (byte)GPR[rt].getUB(0));
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), (byte) (short)(((byte)(GPR[rt]&0xFF))&0xFF));
        }
    };
    
    public Runnable r4300i_SH_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SH", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SH.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), (short)GPR[rt].getUHW(0));
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), (short) (((int)((short)(GPR[rt]&0xFFFF)))&0xFFFF));
        }
    };
    
    public Runnable r4300i_SWL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SWL", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SWL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset) & ~0x03, tmpWord);
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset) & ~0x03, tmpWord);
        }
    };
    
    public Runnable r4300i_SW_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SW", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SW.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), (int)GPR[rt].getUW(0));
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), (int)(GPR[rt]&0x00000000FFFFFFFFL));
        }
    };
    
    public Runnable r4300i_SDL_Debug = new Runnable() {
        public void run() {
            r4300i_SDL.run();
        }
    };
    
    public Runnable r4300i_SDR_Debug = new Runnable() {
        public void run() {
            r4300i_SDR.run();
        }
    };
    
    public Runnable r4300i_SWR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SWR", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SWR.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset) & ~0x03, tmpWord);
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset) & ~0x03, tmpWord);
        }
    };
    
    public Runnable r4300i_CACHE_Debug = new Runnable() {
        public void run() {
            r4300i_CACHE.run();
        }
    };
    
    public Runnable r4300i_LL_Debug = new Runnable() {
        public void run() {
            r4300i_LL.run();
        }
    };
    
    public Runnable r4300i_LWC1_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("LWC1", GPR[base], cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)offset));
            
            r4300i_LWC1.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES || Debug.DEBUG_CP_OPCODES)
//                System.out.printf("ft=%X VA=%X\n", cop1.FPR[cop1.mode32 ? ft>>1 : ft].DW, (int)(GPR[base].getUW(0) + (int)((short)offset)));
                System.out.printf("ft=%X VA=%X\n", cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)((GPR[base]&0x00000000FFFFFFFFL) + (int)offset));
        }
    };
    
    public Runnable r4300i_SC_Debug = new Runnable() {
        public void run() {
            r4300i_SC.run();
        }
    };
    
    public Runnable r4300i_LD_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
                System.out.printf(getITypeDebug("LD", GPR[base], GPR[rt], (int)offset));
            
            r4300i_LD.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES)
//                System.out.printf("rt=%X VA=%X\n", GPR[rt].DW, (int)(GPR[base].getUW(0) + (short)offset));
                System.out.printf("rt=%X VA=%X\n", GPR[rt], (int)((GPR[base]&0x00000000FFFFFFFFL) + offset));
        }
    };
    
    public Runnable r4300i_LDC1_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("LDC1", GPR[base], cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)offset));
            
            r4300i_LDC1.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_LD_OPCODES || Debug.DEBUG_CP_OPCODES)
//                System.out.printf("ft=%X VA=%X\n", cop1.FPR[cop1.mode32 ? ft>>1 : ft].DW, (int)(GPR[base].getUW(0) + (int)((short)offset)));
                System.out.printf("ft=%X VA=%X\n", cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)((GPR[base]&0x00000000FFFFFFFFL) + (int)offset));
        }
    };
    
    public Runnable r4300i_SWC1_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("SWC1", GPR[base], cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)offset));
            
            r4300i_SWC1.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES || Debug.DEBUG_CP_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), cop1.FPR[cop1.mode32 ? ft>>1 : ft].DW);
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW);
        }
    };
    
    public Runnable r4300i_SDC1_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("SDC1", GPR[base], cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW, (int)offset));
            
            r4300i_SDC1.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES || Debug.DEBUG_CP_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), cop1.FPR[cop1.mode32 ? ft>>1 : ft].DW);
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), cop1.FPR[cop1.mode32 ? rt>>1 : rt].DW);
        }
    };
    
    public Runnable r4300i_SD_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
                System.out.printf(getITypeDebug("SD", GPR[base], GPR[rt], (int)offset));
            
            r4300i_SD.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_ST_OPCODES)
//                System.out.printf("VA=%X *=%X\n", (int)(GPR[base].getUW(0) + (short)offset), GPR[rt].DW);
                System.out.printf("VA=%X *=%X\n", (int)((GPR[base]&0x00000000FFFFFFFFL) + offset), GPR[rt]);
        }
    };
    
    /********************** R4300i OpCodes: Special **********************/
    public Runnable r4300i_SPECIAL_SLL_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_SLL.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_SRL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SRL", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SRL.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SRA_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SRA", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SRA.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SLLV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SLLV", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SLLV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SRLV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SRLV", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SRLV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SRAV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SRAV", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SRAV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_JR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getRTypeDebug("JR", GPR[rs], 0, 0));
            
            r4300i_SPECIAL_JR.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_SPECIAL_JALR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getRTypeDebug("JALR", GPR[rs], 0, GPR[rd]));
            
            r4300i_SPECIAL_JALR.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X rd=%X\n", jumpToLocation, GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SYSCALL_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_SYSCALL.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_BREAK_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_BREAK.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_SYNC_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_SYNC.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_MFHI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MFHI", 0, 0, GPR[rd]));
            
            r4300i_SPECIAL_MFHI.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_MTHI_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MTHI", GPR[rs], 0, 0));
            
            r4300i_SPECIAL_MTHI.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("HI=%X\n", HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_MFLO_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MFL0", 0, 0, GPR[rd]));
            
            r4300i_SPECIAL_MFLO.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_MTLO_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MTL0", GPR[rs], 0, 0));
            
            r4300i_SPECIAL_MTLO.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X\n", LO);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSLLV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSLLV", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSLLV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRLV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSRLV", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSRLV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRAV_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DSRAV.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_MULT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MULT", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_MULT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_MULTU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("MULTU", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_MULTU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_DIV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DIV", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_DIV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_DIVU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DIVU", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_DIVU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_DMULT_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DMULT.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_DMULTU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DMULTU", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_DMULTU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_DDIV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DDIV", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_DDIV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_DDIVU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DDIVU", GPR[rs], GPR[rt], 0));
            
            r4300i_SPECIAL_DDIVU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("LO=%X HI=%X\n", LO, HI);
        }
    };
    
    public Runnable r4300i_SPECIAL_ADD_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("ADD", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_ADD.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_ADDU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("ADDU", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_ADDU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SUB_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SUB", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SUB.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SUBU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SUBU", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SUBU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_AND_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("AND", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_AND.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_OR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("OR", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_OR.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_XOR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("XOR", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_XOR.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_NOR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("NOR", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_NOR.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SLT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SLT", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SLT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_SLTU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("SLTU", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_SLTU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DADD_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DADD.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_DADDU_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DADDU", GPR[rs], GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DADDU.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSUB_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DSUB.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_DSUBU_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DSUBU.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_TEQ_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_TEQ.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_DSLL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSLL", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSLL.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSRL", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSRL.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRA_Debug = new Runnable() {
        public void run() {
            r4300i_SPECIAL_DSRA.run();
        }
    };
    
    public Runnable r4300i_SPECIAL_DSLL32_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSLL32", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSLL32.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRL32_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSRL32", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSRL32.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    public Runnable r4300i_SPECIAL_DSRA32_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf(getRTypeDebug("DSRA32", 0, GPR[rt], GPR[rd]));
            
            r4300i_SPECIAL_DSRA32.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_OP_OPCODES)
                System.out.printf("rd=%X\n", GPR[rd]);
        }
    };
    
    /********************** R4300i OpCodes: RegImm **********************/
    public Runnable r4300i_REGIMM_BLTZ_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BLTZ", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_REGIMM_BLTZ.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_REGIMM_BGEZ_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BGEZ", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_REGIMM_BGEZ.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_REGIMM_BLTZL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BLTZL", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_REGIMM_BLTZL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_REGIMM_BGEZL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BGEZL", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_REGIMM_BGEZL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_REGIMM_BLTZAL_Debug = new Runnable() {
        public void run() {
            r4300i_REGIMM_BLTZAL.run();
        }
    };
    
    public Runnable r4300i_REGIMM_BGEZAL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf(getITypeDebug("BGEZAL", GPR[rs], 0, (int)((offset << 2) + 4)));
            
            r4300i_REGIMM_BGEZAL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES)
                System.out.printf("JP=%X 31=%X\n", jumpToLocation, GPR[31]);
        }
    };
    
    /************************** COP0 functions **************************/
    public Runnable r4300i_COP0_MF_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MFC0", 0, GPR[rt], cop0.CP0[rd]&0xFFFFFFFFL));
            
            r4300i_COP0_MF.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_COP0_MT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MTC0", 0, GPR[rt], cop0.CP0[rd]&0xFFFFFFFFL));
            
            r4300i_COP0_MT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("rd=%X\n", cop0.CP0[rd]);
        }
    };
    
    /************************** COP0 CO functions ***********************/
    public Runnable r4300i_COP0_CO_TLBR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("%s.\n", getRTypeDebug("TLBR", 0, 0, 0));
            
            cop0.r4300i_COP0_CO_TLBR.run();
        }
    };
    
    public Runnable r4300i_COP0_CO_TLBWI_Debug = new Runnable() {
        public void run() {
            cop0.r4300i_COP0_CO_TLBWI.run();
        }
    };
    
    public Runnable r4300i_COP0_CO_TLBWR_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("%s.\n", getRTypeDebug("TLBWR", 0, 0, 0));
            
            cop0.r4300i_COP0_CO_TLBWR.run();
        }
    };
    
    public Runnable r4300i_COP0_CO_TLBP_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("%s.\n", getRTypeDebug("TLBP", 0, 0, 0));
            
            cop0.r4300i_COP0_CO_TLBP.run();
        }
    };
    
    public Runnable r4300i_COP0_CO_ERET_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("ERET", 0, 0, 0));
            
            r4300i_COP0_CO_ERET.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    /************************** COP1 functions **************************/
    public Runnable r4300i_COP1_MF_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MFC1", 0, GPR[rt], cop1.FPR[cop1.mode32 ? rd>>1 : rd].DW));
            
            r4300i_COP1_MF.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_COP1_DMF_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("DMFC1", 0, GPR[rt], cop1.FPCR[rd]));
            
            r4300i_COP1_DMF.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_COP1_CF_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CFC1", 0, GPR[rt], cop1.FPCR[rd]));
            
            r4300i_COP1_CF.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("rt=%X\n", GPR[rt]);
        }
    };
    
    public Runnable r4300i_COP1_MT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MTC1", 0, GPR[rt], cop1.FPR[cop1.mode32 ? rd>>1 : rd].DW));
            
            r4300i_COP1_MT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fs=%X\n", cop1.FPR[cop1.mode32 ? rd>>1 : rd].DW);
        }
    };
    
    public Runnable r4300i_COP1_DMT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("DMTC1", 0, GPR[rt], cop1.FPCR[rd]));
            
            r4300i_COP1_DMT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fs=%X\n", cop1.FPR[cop1.mode32 ? rd>>1 : rd].DW);
        }
    };
    
    public Runnable r4300i_COP1_CT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CTC1", 0, GPR[rt], cop1.FPCR[rd]));
            
            r4300i_COP1_CT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fs=%X\n", cop1.FPCR[rd]);
        }
    };
    
    /************************* COP1: BC1 functions ***********************/
    public Runnable r4300i_COP1_BCF_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("BCF", 0, 0, (int)((offset << 2) + 4)));
            
            r4300i_COP1_BCF.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_COP1_BCT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("BCT", 0, 0, (int)((offset << 2) + 4)));
            
            r4300i_COP1_BCT.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_COP1_BCFL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("BCFL", 0, 0, (int)((offset << 2) + 4)));
            
            r4300i_COP1_BCFL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    public Runnable r4300i_COP1_BCTL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getITypeDebug("BCTL", 0, 0, (int)((offset << 2) + 4)));
            
            r4300i_COP1_BCTL.run();
            
            if (Debug.DEBUG_I_OPCODES || Debug.DEBUG_BR_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("JP=%X\n", jumpToLocation);
        }
    };
    
    /************************** COP1: S functions ************************/
    public Runnable r4300i_COP1_S_ADD_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("ADD.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_ADD.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_SUB_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("SUB.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_SUB.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_MUL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MUL.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_MUL.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_DIV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("DIV.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_DIV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_SQRT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("SQRT.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_SQRT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_ABS_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("ABS.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_ABS.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_MOV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MOV.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_MOV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_NEG_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("NEG.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_NEG.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_TRUNC_L_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("TRUNC.L.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_TRUNC_L.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_ROUND_W_Debug = new Runnable() {
        public void run() {
            cop1.r4300i_COP1_S_ROUND_W.run();
        }
    };
    
    public Runnable r4300i_COP1_S_TRUNC_W_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("TRUNC.W.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_TRUNC_W.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_FLOOR_W_Debug = new Runnable() {
        public void run() {
            cop1.r4300i_COP1_S_FLOOR_W.run();
        }
    };
    
    public Runnable r4300i_COP1_S_CVT_D_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.D.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_CVT_D.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_CVT_W_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.W.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_CVT_W.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_CVT_L_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.L.S", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_CVT_L.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_S_CMP_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CMP.S", 0, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_S_CMP.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("31=%X\n", cop1.FPCR[FSTATUS_REGISTER]);
        }
    };
    
    /************************** COP1: D functions ************************/
    public Runnable r4300i_COP1_D_ADD_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("ADD.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_ADD.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_SUB_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("SUB.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_SUB.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_MUL_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MUL.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_MUL.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_DIV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("DIV.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_DIV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_SQRT_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("SQRT.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_SQRT.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_ABS_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("ABS.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_ABS.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_MOV_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("MOV.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_MOV.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_NEG_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("NEG.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_NEG.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_ROUND_W_Debug = new Runnable() {
        public void run() {
            cop1.r4300i_COP1_D_ROUND_W.run();
        }
    };
    
    public Runnable r4300i_COP1_D_TRUNC_W_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("TRUNC.W.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_TRUNC_W.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_CVT_S_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.S.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_CVT_S.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_CVT_W_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.W.D", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_CVT_W.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_D_CVT_L_Debug = new Runnable() {
        public void run() {
            cop1.r4300i_COP1_D_CVT_L.run();
        }
    };
    
    public Runnable r4300i_COP1_D_CMP_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CMP.D", 0, cop1.FPR[cop1.mode32 ? cop1.ft>>1 : cop1.ft].DW, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_D_CMP.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("31=%X\n", cop1.FPCR[FSTATUS_REGISTER]);
        }
    };
    
    /************************** COP1: W functions ************************/
    public Runnable r4300i_COP1_W_CVT_S_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.S.W", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_W_CVT_S.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_W_CVT_D_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.D.W", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_W_CVT_D.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    /************************** COP1: L functions ************************/
    public Runnable r4300i_COP1_L_CVT_S_Debug = new Runnable() {
        public void run() {
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf(getRTypeDebug("CVT.S.L", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW, 0, cop1.FPR[cop1.mode32 ? cop1.fs>>1 : cop1.fs].DW));
            
            cop1.r4300i_COP1_L_CVT_S.run();
            
            if (Debug.DEBUG_R_OPCODES || Debug.DEBUG_CP_OPCODES)
                System.out.printf("fd=%X\n", cop1.FPR[cop1.mode32 ? cop1.fd>>1 : cop1.fd].DW);
        }
    };
    
    public Runnable r4300i_COP1_L_CVT_D_Debug = new Runnable() {
        public void run() {
            cop1.r4300i_COP1_L_CVT_D.run();
        }
    };
    
    /************************** Other functions **************************/
    public Runnable R4300i_UnknownOpcode_Debug = new Runnable() {
        public void run() {
            String Message = String.format("Unhandled R4300i OpCode at: %08X\n%s\n%X\n", pc,
                    R4300iCommands.R4300iOpcodeName(currentInstr, pc), currentInstr);
            Message += "Stoping Emulation !";
            System.out.printf("-%X:Unhandled R4300i OpCode:%X\n", pc, currentInstr);
            System.err.printf("%s\n", Message);
            Main.exit(Message);
        }
    };
    
    public static Runnable[][] buildDebugCpuOps(DebugCpu cpu) {
        Runnable[][] ops = new Runnable[11][0];
        
        ops[Main.R4300I_OPCODE] = new Runnable[64];
        ops[Main.R4300I_SPECIAL] = new Runnable[64];
        ops[Main.R4300I_REGIMM] = new Runnable[32];
        ops[Main.R4300I_COP0] = new Runnable[32];
        ops[Main.R4300I_COP0_FUNCTION] = new Runnable[64];
        ops[Main.R4300I_COP1] = new Runnable[32];
        ops[Main.R4300I_COP1_BC] = new Runnable[32];
        ops[Main.R4300I_COP1_S] = new Runnable[64];
        ops[Main.R4300I_COP1_D] = new Runnable[64];
        ops[Main.R4300I_COP1_W] = new Runnable[64];
        ops[Main.R4300I_COP1_L] = new Runnable[64];
        
        ops[Main.R4300I_OPCODE][ 0] = cpu.R4300i_opcode_SPECIAL;
        ops[Main.R4300I_OPCODE][ 1] = cpu.R4300i_opcode_REGIMM;
        ops[Main.R4300I_OPCODE][ 2] = cpu.r4300i_J_Debug;
        ops[Main.R4300I_OPCODE][ 3] = cpu.r4300i_JAL_Debug;
        ops[Main.R4300I_OPCODE][ 4] = cpu.r4300i_BEQ_Debug;
        ops[Main.R4300I_OPCODE][ 5] = cpu.r4300i_BNE_Debug;
        ops[Main.R4300I_OPCODE][ 6] = cpu.r4300i_BLEZ_Debug;
        ops[Main.R4300I_OPCODE][ 7] = cpu.r4300i_BGTZ_Debug;
        ops[Main.R4300I_OPCODE][ 8] = cpu.r4300i_ADDI_Debug;
        ops[Main.R4300I_OPCODE][ 9] = cpu.r4300i_ADDIU_Debug;
        ops[Main.R4300I_OPCODE][10] = cpu.r4300i_SLTI_Debug;
        ops[Main.R4300I_OPCODE][11] = cpu.r4300i_SLTIU_Debug;
        ops[Main.R4300I_OPCODE][12] = cpu.r4300i_ANDI_Debug;
        ops[Main.R4300I_OPCODE][13] = cpu.r4300i_ORI_Debug;
        ops[Main.R4300I_OPCODE][14] = cpu.r4300i_XORI_Debug;
        ops[Main.R4300I_OPCODE][15] = cpu.r4300i_LUI_Debug;
        ops[Main.R4300I_OPCODE][16] = cpu.R4300i_opcode_COP0;
        ops[Main.R4300I_OPCODE][17] = cpu.R4300i_opcode_COP1;
        ops[Main.R4300I_OPCODE][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][20] = cpu.r4300i_BEQL_Debug;
        ops[Main.R4300I_OPCODE][21] = cpu.r4300i_BNEL_Debug;
        ops[Main.R4300I_OPCODE][22] = cpu.r4300i_BLEZL_Debug;
        ops[Main.R4300I_OPCODE][23] = cpu.r4300i_BGTZL_Debug;
        ops[Main.R4300I_OPCODE][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][25] = cpu.r4300i_DADDIU_Debug;
        ops[Main.R4300I_OPCODE][26] = cpu.r4300i_LDL_Debug;
        ops[Main.R4300I_OPCODE][27] = cpu.r4300i_LDR_Debug;
        ops[Main.R4300I_OPCODE][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][32] = cpu.r4300i_LB_Debug;
        ops[Main.R4300I_OPCODE][33] = cpu.r4300i_LH_Debug;
        ops[Main.R4300I_OPCODE][34] = cpu.r4300i_LWL_Debug;
        ops[Main.R4300I_OPCODE][35] = cpu.r4300i_LW_Debug;
        ops[Main.R4300I_OPCODE][36] = cpu.r4300i_LBU_Debug;
        ops[Main.R4300I_OPCODE][37] = cpu.r4300i_LHU_Debug;
        ops[Main.R4300I_OPCODE][38] = cpu.r4300i_LWR_Debug;
        ops[Main.R4300I_OPCODE][39] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_LWU;
        ops[Main.R4300I_OPCODE][40] = cpu.r4300i_SB_Debug;
        ops[Main.R4300I_OPCODE][41] = cpu.r4300i_SH_Debug;
        ops[Main.R4300I_OPCODE][42] = cpu.r4300i_SWL_Debug;
        ops[Main.R4300I_OPCODE][43] = cpu.r4300i_SW_Debug;
        ops[Main.R4300I_OPCODE][44] = cpu.r4300i_SDL_Debug;
        ops[Main.R4300I_OPCODE][45] = cpu.r4300i_SDR_Debug;
        ops[Main.R4300I_OPCODE][46] = cpu.r4300i_SWR_Debug;
        ops[Main.R4300I_OPCODE][47] = cpu.r4300i_CACHE_Debug;
        ops[Main.R4300I_OPCODE][48] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_LL;
        ops[Main.R4300I_OPCODE][49] = cpu.r4300i_LWC1_Debug;
        ops[Main.R4300I_OPCODE][50] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][51] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][52] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][53] = cpu.r4300i_LDC1_Debug;
        ops[Main.R4300I_OPCODE][54] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][55] = cpu.r4300i_LD_Debug;
        ops[Main.R4300I_OPCODE][56] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SC;
        ops[Main.R4300I_OPCODE][57] = cpu.r4300i_SWC1_Debug;
        ops[Main.R4300I_OPCODE][58] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][59] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][60] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][61] = cpu.r4300i_SDC1_Debug;
        ops[Main.R4300I_OPCODE][62] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_OPCODE][63] = cpu.r4300i_SD_Debug;
        
        ops[Main.R4300I_SPECIAL][ 0] = cpu.r4300i_SPECIAL_SLL_Debug; // NOP?
        ops[Main.R4300I_SPECIAL][ 1] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][ 2] = cpu.r4300i_SPECIAL_SRL_Debug;
        ops[Main.R4300I_SPECIAL][ 3] = cpu.r4300i_SPECIAL_SRA_Debug;
        ops[Main.R4300I_SPECIAL][ 4] = cpu.r4300i_SPECIAL_SLLV_Debug;
        ops[Main.R4300I_SPECIAL][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][ 6] = cpu.r4300i_SPECIAL_SRLV_Debug;
        ops[Main.R4300I_SPECIAL][ 7] = cpu.r4300i_SPECIAL_SRAV_Debug;
        ops[Main.R4300I_SPECIAL][ 8] = cpu.r4300i_SPECIAL_JR_Debug;
        ops[Main.R4300I_SPECIAL][ 9] = cpu.r4300i_SPECIAL_JALR_Debug;
        ops[Main.R4300I_SPECIAL][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][12] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_SYSCALL;
        ops[Main.R4300I_SPECIAL][13] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_BREAK; // ??
        ops[Main.R4300I_SPECIAL][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][15] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_SYNC;
        ops[Main.R4300I_SPECIAL][16] = cpu.r4300i_SPECIAL_MFHI_Debug;
        ops[Main.R4300I_SPECIAL][17] = cpu.r4300i_SPECIAL_MTHI_Debug;
        ops[Main.R4300I_SPECIAL][18] = cpu.r4300i_SPECIAL_MFLO_Debug;
        ops[Main.R4300I_SPECIAL][19] = cpu.r4300i_SPECIAL_MTLO_Debug;
        ops[Main.R4300I_SPECIAL][20] = cpu.r4300i_SPECIAL_DSLLV_Debug;
        ops[Main.R4300I_SPECIAL][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][22] = cpu.r4300i_SPECIAL_DSRLV_Debug;
        ops[Main.R4300I_SPECIAL][23] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DSRAV;
        ops[Main.R4300I_SPECIAL][24] = cpu.r4300i_SPECIAL_MULT_Debug;
        ops[Main.R4300I_SPECIAL][25] = cpu.r4300i_SPECIAL_MULTU_Debug;
        ops[Main.R4300I_SPECIAL][26] = cpu.r4300i_SPECIAL_DIV_Debug;
        ops[Main.R4300I_SPECIAL][27] = cpu.r4300i_SPECIAL_DIVU_Debug;
        ops[Main.R4300I_SPECIAL][28] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DMULT;
        ops[Main.R4300I_SPECIAL][29] = cpu.r4300i_SPECIAL_DMULTU_Debug;
        ops[Main.R4300I_SPECIAL][30] = cpu.r4300i_SPECIAL_DDIV_Debug;
        ops[Main.R4300I_SPECIAL][31] = cpu.r4300i_SPECIAL_DDIVU_Debug;
        ops[Main.R4300I_SPECIAL][32] = cpu.r4300i_SPECIAL_ADD_Debug;
        ops[Main.R4300I_SPECIAL][33] = cpu.r4300i_SPECIAL_ADDU_Debug;
        ops[Main.R4300I_SPECIAL][34] = cpu.r4300i_SPECIAL_SUB_Debug;
        ops[Main.R4300I_SPECIAL][35] = cpu.r4300i_SPECIAL_SUBU_Debug;
        ops[Main.R4300I_SPECIAL][36] = cpu.r4300i_SPECIAL_AND_Debug;
        ops[Main.R4300I_SPECIAL][37] = cpu.r4300i_SPECIAL_OR_Debug;
        ops[Main.R4300I_SPECIAL][38] = cpu.r4300i_SPECIAL_XOR_Debug;
        ops[Main.R4300I_SPECIAL][39] = cpu.r4300i_SPECIAL_NOR_Debug;
        ops[Main.R4300I_SPECIAL][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][42] = cpu.r4300i_SPECIAL_SLT_Debug;
        ops[Main.R4300I_SPECIAL][43] = cpu.r4300i_SPECIAL_SLTU_Debug;
        ops[Main.R4300I_SPECIAL][44] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DADD;
        ops[Main.R4300I_SPECIAL][45] = cpu.r4300i_SPECIAL_DADDU_Debug;
        ops[Main.R4300I_SPECIAL][46] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DSUB;
        ops[Main.R4300I_SPECIAL][47] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DSUBU;
        ops[Main.R4300I_SPECIAL][48] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][49] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][50] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][51] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][52] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_TEQ;
        ops[Main.R4300I_SPECIAL][53] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][54] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][55] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][56] = cpu.r4300i_SPECIAL_DSLL_Debug;
        ops[Main.R4300I_SPECIAL][57] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][58] = cpu.r4300i_SPECIAL_DSRL_Debug;
        ops[Main.R4300I_SPECIAL][59] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_SPECIAL_DSRA;
        ops[Main.R4300I_SPECIAL][60] = cpu.r4300i_SPECIAL_DSLL32_Debug;
        ops[Main.R4300I_SPECIAL][61] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_SPECIAL][62] = cpu.r4300i_SPECIAL_DSRL32_Debug;
        ops[Main.R4300I_SPECIAL][63] = cpu.r4300i_SPECIAL_DSRA32_Debug;
        
        ops[Main.R4300I_REGIMM][ 0] = cpu.r4300i_REGIMM_BLTZ_Debug;
        ops[Main.R4300I_REGIMM][ 1] = cpu.r4300i_REGIMM_BGEZ_Debug;
        ops[Main.R4300I_REGIMM][ 2] = cpu.r4300i_REGIMM_BLTZL_Debug;
        ops[Main.R4300I_REGIMM][ 3] = cpu.r4300i_REGIMM_BGEZL_Debug;
        ops[Main.R4300I_REGIMM][ 4] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][ 6] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][16] = cpu.r4300i_REGIMM_BLTZAL_Debug;
        ops[Main.R4300I_REGIMM][17] = cpu.r4300i_REGIMM_BGEZAL_Debug;
        ops[Main.R4300I_REGIMM][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_REGIMM][31] = cpu.R4300i_UnknownOpcode_Debug;
        
        ops[Main.R4300I_COP0][ 0] = cpu.r4300i_COP0_MF_Debug;
        ops[Main.R4300I_COP0][ 1] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 2] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 3] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 4] = cpu.r4300i_COP0_MT_Debug;
        ops[Main.R4300I_COP0][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 6] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0][16] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][17] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][18] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][19] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][20] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][21] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][22] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][23] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][24] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][25] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][26] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][27] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][28] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][29] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][30] = cpu.R4300i_opcode_COP0_CO;
        ops[Main.R4300I_COP0][31] = cpu.R4300i_opcode_COP0_CO;
        
        ops[Main.R4300I_COP0_FUNCTION][ 0] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 1] = cpu.r4300i_COP0_CO_TLBR_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 2] = cpu.r4300i_COP0_CO_TLBWI_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 3] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 4] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 6] = cpu.r4300i_COP0_CO_TLBWR_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 8] = cpu.r4300i_COP0_CO_TLBP_Debug;
        ops[Main.R4300I_COP0_FUNCTION][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][24] = cpu.r4300i_COP0_CO_ERET_Debug;
        ops[Main.R4300I_COP0_FUNCTION][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][32] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][33] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][34] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][35] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][36] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][37] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][38] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][39] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][42] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][43] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][44] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][45] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][46] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][47] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][48] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][49] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][50] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][51] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][52] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][53] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][54] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][55] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][56] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][57] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][58] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][59] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][60] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][61] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][62] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP0_FUNCTION][63] = cpu.R4300i_UnknownOpcode_Debug;
        
        ops[Main.R4300I_COP1][ 0] = cpu.r4300i_COP1_MF_Debug;
        ops[Main.R4300I_COP1][ 1] = cpu.r4300i_COP1_DMF_Debug;
        ops[Main.R4300I_COP1][ 2] = cpu.r4300i_COP1_CF_Debug;
        ops[Main.R4300I_COP1][ 3] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][ 4] = cpu.r4300i_COP1_MT_Debug;
        ops[Main.R4300I_COP1][ 5] = cpu.r4300i_COP1_DMT_Debug;
        ops[Main.R4300I_COP1][ 6] = cpu.r4300i_COP1_CT_Debug;
        ops[Main.R4300I_COP1][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][ 8] = cpu.R4300i_opcode_COP1_BC;
        ops[Main.R4300I_COP1][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][16] = cpu.R4300i_opcode_COP1_S;
        ops[Main.R4300I_COP1][17] = cpu.R4300i_opcode_COP1_D;
        ops[Main.R4300I_COP1][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][20] = cpu.R4300i_opcode_COP1_W;
        ops[Main.R4300I_COP1][21] = cpu.R4300i_opcode_COP1_L;
        ops[Main.R4300I_COP1][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1][31] = cpu.R4300i_UnknownOpcode_Debug;
        
        ops[Main.R4300I_COP1_BC][ 0] = cpu.r4300i_COP1_BCF_Debug;
        ops[Main.R4300I_COP1_BC][ 1] = cpu.r4300i_COP1_BCT_Debug;
        ops[Main.R4300I_COP1_BC][ 2] = cpu.r4300i_COP1_BCFL_Debug;
        ops[Main.R4300I_COP1_BC][ 3] = cpu.r4300i_COP1_BCTL_Debug;
        ops[Main.R4300I_COP1_BC][ 4] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][ 6] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_BC][31] = cpu.R4300i_UnknownOpcode_Debug;
        
        ops[Main.R4300I_COP1_S][ 0] = cpu.r4300i_COP1_S_ADD_Debug;
        ops[Main.R4300I_COP1_S][ 1] = cpu.r4300i_COP1_S_SUB_Debug;
        ops[Main.R4300I_COP1_S][ 2] = cpu.r4300i_COP1_S_MUL_Debug;
        ops[Main.R4300I_COP1_S][ 3] = cpu.r4300i_COP1_S_DIV_Debug;
        ops[Main.R4300I_COP1_S][ 4] = cpu.r4300i_COP1_S_SQRT_Debug;
        ops[Main.R4300I_COP1_S][ 5] = cpu.r4300i_COP1_S_ABS_Debug;
        ops[Main.R4300I_COP1_S][ 6] = cpu.r4300i_COP1_S_MOV_Debug;
        ops[Main.R4300I_COP1_S][ 7] = cpu.r4300i_COP1_S_NEG_Debug;
        ops[Main.R4300I_COP1_S][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][ 9] = cpu.r4300i_COP1_S_TRUNC_L_Debug;
        ops[Main.R4300I_COP1_S][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][12] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_ROUND_W;
        ops[Main.R4300I_COP1_S][13] = cpu.r4300i_COP1_S_TRUNC_W_Debug;
        ops[Main.R4300I_COP1_S][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][15] = cpu.r4300i_COP1_S_FLOOR_W_Debug;
        ops[Main.R4300I_COP1_S][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][32] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][33] = cpu.r4300i_COP1_S_CVT_D_Debug;
        ops[Main.R4300I_COP1_S][34] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][35] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][36] = cpu.r4300i_COP1_S_CVT_W_Debug;
        ops[Main.R4300I_COP1_S][37] = cpu.r4300i_COP1_S_CVT_L_Debug;
        ops[Main.R4300I_COP1_S][38] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][39] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][42] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][43] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][44] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][45] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][46] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][47] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_S][48] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][49] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][50] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][51] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][52] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][53] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][54] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][55] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][56] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][57] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][58] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][59] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][60] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][61] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        ops[Main.R4300I_COP1_S][62] = cpu.r4300i_COP1_S_CMP_Debug;
        ops[Main.R4300I_COP1_S][63] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_S_CMP;
        
        ops[Main.R4300I_COP1_D][ 0] = cpu.r4300i_COP1_D_ADD_Debug;
        ops[Main.R4300I_COP1_D][ 1] = cpu.r4300i_COP1_D_SUB_Debug;
        ops[Main.R4300I_COP1_D][ 2] = cpu.r4300i_COP1_D_MUL_Debug;
        ops[Main.R4300I_COP1_D][ 3] = cpu.r4300i_COP1_D_DIV_Debug;
        ops[Main.R4300I_COP1_D][ 4] = cpu.r4300i_COP1_D_SQRT_Debug;
        ops[Main.R4300I_COP1_D][ 5] = cpu.r4300i_COP1_D_ABS_Debug;
        ops[Main.R4300I_COP1_D][ 6] = cpu.r4300i_COP1_D_MOV_Debug;
        ops[Main.R4300I_COP1_D][ 7] = cpu.r4300i_COP1_D_NEG_Debug;
        ops[Main.R4300I_COP1_D][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][12] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_ROUND_W;
        ops[Main.R4300I_COP1_D][13] = cpu.r4300i_COP1_D_TRUNC_W_Debug;
        ops[Main.R4300I_COP1_D][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][32] = cpu.r4300i_COP1_D_CVT_S_Debug;
        ops[Main.R4300I_COP1_D][33] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][34] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][35] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][36] = cpu.r4300i_COP1_D_CVT_W_Debug;
        ops[Main.R4300I_COP1_D][37] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CVT_L;
        ops[Main.R4300I_COP1_D][38] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][39] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][42] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][43] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][44] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][45] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][46] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][47] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_D][48] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][49] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][50] = cpu.r4300i_COP1_D_CMP_Debug;
        ops[Main.R4300I_COP1_D][51] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][52] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][53] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][54] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][55] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][56] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][57] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][58] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][59] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][60] = cpu.r4300i_COP1_D_CMP_Debug;
        ops[Main.R4300I_COP1_D][61] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        ops[Main.R4300I_COP1_D][62] = cpu.r4300i_COP1_D_CMP_Debug;
        ops[Main.R4300I_COP1_D][63] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_D_CMP;
        
        ops[Main.R4300I_COP1_W][ 0] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 1] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 2] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 3] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 4] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 6] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][32] = cpu.r4300i_COP1_W_CVT_S_Debug;
        ops[Main.R4300I_COP1_W][33] = cpu.r4300i_COP1_W_CVT_D_Debug;
        ops[Main.R4300I_COP1_W][34] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][35] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][36] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][37] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][38] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][39] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][42] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][43] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][44] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][45] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][46] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][47] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][48] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][49] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][50] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][51] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][52] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][53] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][54] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][55] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][56] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][57] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][58] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][59] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][60] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][61] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][62] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_W][63] = cpu.R4300i_UnknownOpcode_Debug;
        
        ops[Main.R4300I_COP1_L][ 0] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 1] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 2] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 3] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 4] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 5] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 6] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 7] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 8] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][ 9] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][10] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][11] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][12] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][13] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][14] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][15] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][16] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][17] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][18] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][19] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][20] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][21] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][22] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][23] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][24] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][25] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][26] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][27] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][28] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][29] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][30] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][31] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][32] = cpu.r4300i_COP1_L_CVT_S_Debug;
        ops[Main.R4300I_COP1_L][33] = cpu.R4300i_UnknownOpcode_Debug; //cpu.r4300i_COP1_L_CVT_D;
        ops[Main.R4300I_COP1_L][34] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][35] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][36] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][37] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][38] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][39] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][40] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][41] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][42] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][43] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][44] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][45] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][46] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][47] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][48] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][49] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][50] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][51] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][52] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][53] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][54] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][55] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][56] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][57] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][58] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][59] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][60] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][61] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][62] = cpu.R4300i_UnknownOpcode_Debug;
        ops[Main.R4300I_COP1_L][63] = cpu.R4300i_UnknownOpcode_Debug;
        
        return ops;
    }
    
}
